from django.db.models import Sum, FloatField, Q
from django.db.models.functions import TruncMonth
from django.http import JsonResponse
from django.views import View
from django.views.generic.list import MultipleObjectMixin

from main.models import Account, Category, Snapshot, Transaction
from main.views import UserMixin


class TransactionListView(UserMixin, MultipleObjectMixin, View):
    model = Transaction

    def get(self, request, *args, **kwargs):
        return JsonResponse({"transactions": list(self.get_queryset().values())})


class CategoryListView(UserMixin, MultipleObjectMixin, View):
    model = Category

    def get(self, request, *args, **kwargs):
        return JsonResponse({"categories": list(self.get_queryset().values())})


class AccountListView(UserMixin, MultipleObjectMixin, View):
    model = Account

    def get(self, request, *args, **kwargs):
        return JsonResponse({"accounts": list(self.get_queryset().values())})


class SnapshotListView(UserMixin, MultipleObjectMixin, View):
    model = Snapshot

    def get(self, request, *args, **kwargs):
        return JsonResponse({"snapshots": list(self.get_queryset().values())})


class HistoryView(UserMixin, MultipleObjectMixin, View):
    model = Transaction

    def get(self, request, *args, **kwargs):
        return JsonResponse(
            {
                "data": list(
                    self.get_queryset()
                    .filter(category__budget=True)
                    .values(month=TruncMonth("date"))
                    .annotate(
                        sum_p=Sum(
                            "value", output_field=FloatField(), filter=Q(value__gt=0)
                        ),
                        sum_m=Sum(
                            "value", output_field=FloatField(), filter=Q(value__lt=0)
                        ),
                        sum=Sum("value", output_field=FloatField()),
                    )
                    .order_by("month")
                )
            }
        )
